@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bftextview2_autocomp.c
  *
- * Copyright (C) 2008,2009,2010,2011,2012,2013 Olivier Sessink
+ * Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +37,8 @@ typedef struct {
 	GtkTreeView *tree;
 	GtkWidget *scroll;
 	GtkWidget *reflabel;
+	GtkCellRenderer *cell;
+	GtkTreeViewColumn *column;
 	gint listwidth;
 	gint w;
 	gint h;
@@ -189,6 +191,12 @@ get_existing_end_len(BluefishTextView * btv, const gchar *string, gint prefix_by
 	return 0;
 }
 
+static gboolean trigger_new_autocomp_idle(gpointer data) {
+	DBG_AUTOCOMP("trigger_new_autocomp_idle\n");
+	autocomp_run(BLUEFISH_TEXT_VIEW(data), FALSE);
+	return FALSE;
+}
+
 gboolean
 acwin_check_keypress(BluefishTextView * btv, GdkEventKey * event)
 {
@@ -210,6 +218,7 @@ acwin_check_keypress(BluefishTextView * btv, GdkEventKey * event)
 				gint existing_len, prefix_len;
 				guint pattern_id;
 				gint backup_chars = 0;
+				gboolean trigger_new_autocomp_popup=FALSE;
 				gtk_tree_model_get(model, &it, 1, &string, -1);
 				/*g_print("context %d has patternhash %p, string=%s\n",ACWIN(btv->autocomp)->contextnum, g_array_index(btv->bflang->st->contexts, Tcontext, ACWIN(btv->autocomp)->contextnum).patternhash, string); */
 				if (g_array_index
@@ -224,12 +233,13 @@ acwin_check_keypress(BluefishTextView * btv, GdkEventKey * event)
 					if (pattern_id) {
 						GSList *tmpslist =
 							g_array_index(master->bflang->st->matches, Tpattern, pattern_id).autocomp_items;
-						/* a pattern MAY have multiple autocomplete items. This code is not efficient iof in the future some
+						/* a pattern MAY have multiple autocomplete items. This code is not efficient if in the future some
 						   patterns would have many autocomplete items. I don't expect this, so I leave this as it is right now  */
 						while (tmpslist) {
 							Tpattern_autocomplete *pac = tmpslist->data;
 							if (g_strcmp0(string, pac->autocomplete_string) == 0) {
 								backup_chars = pac->autocomplete_backup_cursor;
+								trigger_new_autocomp_popup = (pac->trigger_new_autocomp_popup==1);
 							}
 							tmpslist = g_slist_next(tmpslist);
 						}
@@ -253,7 +263,9 @@ acwin_check_keypress(BluefishTextView * btv, GdkEventKey * event)
 						gtk_text_buffer_place_cursor(btv->buffer, &iter);
 					}
 				}
-
+				if (trigger_new_autocomp_popup) {
+					g_idle_add(trigger_new_autocomp_idle, btv);
+				}
 				g_free(string);
 			}
 			acwin_cleanup(btv);
@@ -314,10 +326,12 @@ acw_selection_changed_lcb(GtkTreeSelection * selection, Tacwin * acw)
 		gchar *key;
 		gtk_tree_model_get(model, &iter, 1, &key, -1);
 		if (key) {
+			DBG_AUTOCOMP("lookup reference for %s\n",key);
 			gint pattern_id =
 				GPOINTER_TO_INT(g_hash_table_lookup
 								(g_array_index
 								 (master->bflang->st->contexts, Tcontext, acw->contextnum).patternhash, key));
+			DBG_AUTOCOMP("got pattern_id %d for key %s\n",pattern_id, key);
 			g_free(key);
 			if (pattern_id && g_array_index(master->bflang->st->matches, Tpattern, pattern_id).reference) {
 				GtkRequisition requisition;
@@ -334,7 +348,7 @@ acw_selection_changed_lcb(GtkTreeSelection * selection, Tacwin * acw)
 #endif
 				/*gtk_window_get_size(GTK_WINDOW(acw->win),&width,&height); */
 				acw->w = acw->listwidth + requisition.width + 2;
-				gtk_widget_set_size_request(acw->win, acw->w, -1);
+				gtk_widget_set_size_request(acw->win, acw->w, (acw->h<350)?350:-1);
 				return;
 			}
 		}
@@ -347,8 +361,7 @@ acw_selection_changed_lcb(GtkTreeSelection * selection, Tacwin * acw)
 static Tacwin *
 acwin_create(BluefishTextView * btv)
 {
-	GtkCellRenderer *cell;
-	GtkTreeViewColumn *column;
+	/*GtkCellRenderer *cell;*/
 	GtkWidget *hbox;
 	/* GtkWidget *vbar; */
 	Tacwin *acw;
@@ -358,7 +371,13 @@ acwin_create(BluefishTextView * btv)
 	acw->btv = btv;
 	acw->in_fill = TRUE;
 	acw->win = gtk_window_new(GTK_WINDOW_POPUP);
+/* We do not do any customized drawing in autocomplete window, as far as I can see, so probably we do not need to switch this on.
+* On MacOSX this causes the labels of acw->reflabel to pile up on each other, see report by Keith Gunthardt on Facebook:
+* https://www.facebook.com/photo.php?fbid=10152794950839751&set=p.10152794950839751&type=1 
+
+	I (Olivier) think you are right Andrius, we can remove this call for all platforms, uncommenting it:
 	gtk_widget_set_app_paintable(acw->win, TRUE);
+*/
 	gtk_window_set_resizable(GTK_WINDOW(acw->win), FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER(acw->win), 1);
 	gtk_window_set_decorated(GTK_WINDOW(acw->win), FALSE);
@@ -371,9 +390,9 @@ acwin_create(BluefishTextView * btv)
 	gtk_tree_view_set_headers_visible(acw->tree, FALSE);
 	acw->scroll = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(acw->scroll), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	cell = gtk_cell_renderer_text_new();
-	column = gtk_tree_view_column_new_with_attributes("", cell, "markup", 0, NULL);
-	gtk_tree_view_append_column(GTK_TREE_VIEW(acw->tree), column);
+	acw->cell = gtk_cell_renderer_text_new();
+	acw->column = gtk_tree_view_column_new_with_attributes("", acw->cell, "markup", 0, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(acw->tree), acw->column);
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(acw->tree), FALSE);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(acw->tree));
 	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(acw_selection_changed_lcb), acw);
@@ -421,12 +440,16 @@ acwin_position_at_cursor(BluefishTextView * btv)
 	gdk_window_get_origin(gtk_text_view_get_window(GTK_TEXT_VIEW(btv), GTK_TEXT_WINDOW_TEXT), &x, &y);
 
 	sh = gdk_screen_get_height(screen);
-	DBG_AUTOCOMP("rect.y+y=%d, acw->h=%d, sh=%d\n", rect.y + y, ACWIN(btv->autocomp)->h, sh);
 	if (rect.y + y + ACWIN(btv->autocomp)->h > sh) {
+		DBG_AUTOCOMP("acwin_position_at_cursor, popup above cursuor, rect.y+y=%d + rect.height(%d)-acw->h(%d)=%d, acw->h=%d, sh=%d\n"
+				, rect.y + y,rect.height,ACWIN(btv->autocomp)->h
+				, rect.y + y + rect.height - ACWIN(btv->autocomp)->h
+				, ACWIN(btv->autocomp)->h, sh);
 		gtk_window_move(GTK_WINDOW(ACWIN(btv->autocomp)->win), rect.x + x,
 						rect.y + y + rect.height - ACWIN(btv->autocomp)->h);
 		return FALSE;
 	} else {
+		DBG_AUTOCOMP("acwin_position_at_cursor, popup below cursuor, rect.y+y=%d, acw->h=%d, sh=%d\n", rect.y + y, ACWIN(btv->autocomp)->h, sh);
 		gtk_window_move(GTK_WINDOW(ACWIN(btv->autocomp)->win), rect.x + x, rect.y + y);
 		return TRUE;
 	}
@@ -456,9 +479,9 @@ acwin_calculate_window_size(Tacwin * acw, GList * items, GList * items2, const g
 		}
 		g_assert(tmplist != NULL);
 		g_assert(tmplist->data != NULL);
-		DBG_AUTOCOMP("acwin_calculate_window_size, tmplist=%p", tmplist);
+/*		DBG_AUTOCOMP("acwin_calculate_window_size, tmplist=%p", tmplist);
 		DBG_AUTOCOMP(", tmplist->data=%p",tmplist->data);
-		DBG_AUTOCOMP("=%s\n", (gchar *)tmplist->data);
+		DBG_AUTOCOMP("=%s\n", (gchar *)tmplist->data);*/
 		tmp = g_markup_escape_text(tmplist->data, -1);
 		len = strlen(tmp);
 		if (len > longestlen) {
@@ -471,14 +494,30 @@ acwin_calculate_window_size(Tacwin * acw, GList * items, GList * items2, const g
 		tmplist = g_list_next(tmplist);
 	}
 	if (longest) {
-		gint len, rowh;
+		gint len, rowh,min_h,nat_h;
 		PangoLayout *panlay = gtk_widget_create_pango_layout(GTK_WIDGET(acw->tree), NULL);
-		DBG_AUTOCOMP("longest=%s\n", longest);
 		pango_layout_set_markup(panlay, longest, -1);
 		pango_layout_get_pixel_size(panlay, &len, &rowh);
-		acw->h = MIN(MAX((*numitems + 1) * rowh + 8, 150), 350);
+		DBG_AUTOCOMP("longest=%s which has len=%d and rowheight=%d\n", longest, len, rowh);
+		/*  rowh+9 has been found by trial and error on a particular gtk theme and resolution, so
+		there is quite a chance that this will not be optimal for other themes or for example
+		on a high resolution display. I've tried to request this size but I cannot find it.
+		I've now changed it to nat_h+5, hopefully nat_h is already a more dynamic value */
+		/*gint spacing,height;
+		GtkRequisition min_size, nat_size;
+		gtk_cell_renderer_get_preferred_height(acw->cell,GTK_WIDGET(acw->tree),&min_h,&nat_h);
+		spacing= gtk_tree_view_column_get_spacing(acw->column);
+		gtk_tree_view_column_cell_get_size(acw->column,NULL,NULL,NULL,NULL,&height);
+		gtk_widget_get_preferred_size(GTK_WIDGET(acw->tree),&min_size,&nat_size);
+		g_print("row=%d,rowh+9=%d,min_h=%d,nat_h=%d,spacing=%d,height=%d,nat_size.h=%d\n",rowh,rowh+9,min_h,nat_h,spacing,height,min_size.height);*/
+#if GTK_CHECK_VERSION(3,0,0)
+		gtk_cell_renderer_get_preferred_height(acw->cell,GTK_WIDGET(acw->tree),&min_h,&nat_h);
+#else
+		nat_h=rowh+2;
+#endif
+		acw->h = MIN((*numitems) * (nat_h+5), 350); /*MIN(MAX((*numitems + 1) * rowh + 8, 150), 350);*/
 		acw->w = acw->listwidth = MIN(len + 20, 350);
-		DBG_AUTOCOMP("numitems=%d, rowh=%d, new height=%d, listwidth=%d\n", *numitems, rowh, acw->h,
+		DBG_AUTOCOMP("acwin_calculate_window_size, numitems=%d, rowh=%d, new height=%d, new width=%d\n", *numitems, rowh, acw->h,
 				acw->listwidth);
 		gtk_widget_set_size_request(GTK_WIDGET(acw->scroll), acw->listwidth, acw->h);	/* ac_window */
 		g_free(longest);
@@ -538,7 +577,7 @@ acwin_fill_tree(Tacwin * acw, GList * items, GList * items2, gchar * closetag, g
 			gtk_list_store_append(acw->store, &it);
 			tmp = g_markup_escape_text(tmplist->data, -1);
 			gtk_list_store_set(acw->store, &it, 0, tmp, 1, tmplist->data, -1);
-			DBG_AUTOCOMP("acwin_fill_tree, add item %s\n",tmp);
+			/*DBG_AUTOCOMP("acwin_fill_tree, add item %s\n",tmp);*/
 			g_free(tmp);
 		}
 		i++;
@@ -563,6 +602,47 @@ autocomp_stop(BluefishTextView * btv)
 	acwin_cleanup(btv);
 }
 
+static GList *
+process_conditional_items(BluefishTextView * btv, Tfound *found, gint contextnum, GList *items)
+{
+	BluefishTextView *master = BLUEFISH_TEXT_VIEW(btv->master);
+	GList *retval=NULL, *tmplist=items;
+	
+	while (tmplist) {
+		gboolean valid=TRUE;
+		guint pattern_id;
+		const gchar *string = tmplist->data;
+		GHashTable *hasht = g_array_index(master->bflang->st->contexts, Tcontext,contextnum).patternhash;
+		g_print("process_conditional_items, looking up %s in hashtable %p\n",string, hasht);
+		pattern_id = GPOINTER_TO_INT(g_hash_table_lookup(hasht, string));
+		DBG_AUTOCOMP("process_conditional_items, got pattern_id=%d\n", pattern_id);
+		if (pattern_id && found) {
+				/* found may be NULL, for examplein an empty document, in that case anything is valid */
+			GSList *tmpslist = g_array_index(master->bflang->st->matches, Tpattern, pattern_id).autocomp_items;
+			/* a pattern MAY have multiple autocomplete items. This code is not efficient if in the future some
+			   patterns would have many autocomplete items. I don't expect this, so I leave this as it is right now  */
+			while (tmpslist) {
+				Tpattern_autocomplete *pac = tmpslist->data;
+				if (g_strcmp0(string, pac->autocomplete_string) == 0 && pac->condition!=0) {
+					Tpattern_condition *pcond = &g_array_index(master->bflang->st->conditions, Tpattern_condition, pac->condition);
+					
+					valid = test_condition(found->fcontext, found->fblock, pcond);
+					if (!valid) {
+						g_print("process_conditional_items, item %s is NOT VALID for autocompletion\n",string);
+					}
+				}
+				tmpslist = g_slist_next(tmpslist);
+			}
+		}
+		if (valid) {
+			retval = g_list_prepend(retval, string);
+		}
+		
+		tmplist = tmplist->next;
+	}
+	return retval;
+}
+
 void
 autocomp_run(BluefishTextView * btv, gboolean user_requested)
 {
@@ -570,7 +650,11 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 	BluefishTextView *master = BLUEFISH_TEXT_VIEW(btv->master);
 	gint contextnum;
 	gunichar uc;
-	Tfoundblock *fblock = NULL;	/* needed for the special case to close generix xml tags based on the top of the blockstack */
+
+	Tfound *found=NULL;
+	Tfoundblock *fblock = NULL;	/* needed for the special case to close generix xml tags 
+											based on the top of the blockstack, or to match conditional 
+											autocompletion strings */
 
 	if (G_UNLIKELY(!master->bflang || !master->bflang->st))
 		return;
@@ -607,9 +691,7 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 	}
 
 	if (g_array_index(master->bflang->st->contexts, Tcontext, contextnum).has_tagclose_from_blockstack) {
-		Tfound *found;
-		GSequenceIter *siter = NULL;
-		found = get_foundcache_at_offset(master, gtk_text_iter_get_offset(&cursorpos), &siter);
+		found = get_foundcache_at_offset(btv, gtk_text_iter_get_offset(&cursorpos), NULL);
 		if (found) {
 			fblock =
 				found->numblockchange < 0 ? pop_blocks(found->numblockchange, found->fblock) : found->fblock;
@@ -638,6 +720,7 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 		/* we have a prefix or it is user requested, and we have a context with autocompletion or we have blockstack-tag-auto-closing */
 		gchar *newprefix = NULL, *prefix, *closetag = NULL;
 		GList *items = NULL, *items2 = NULL;
+		gboolean free_items=FALSE;
 		/*print_ac_items(g_array_index(btv->bflang->st->contexts,Tcontext, contextnum).ac); */
 
 		prefix = gtk_text_buffer_get_text(btv->buffer, &iter, &cursorpos, TRUE);
@@ -670,7 +753,13 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 									  prefix, &newprefix);
 			DBG_AUTOCOMP("got %d autocompletion items for prefix %s in context %d, newprefix=%s\n",
 						 g_list_length(items), prefix, contextnum, newprefix);
-#ifdef IDENTSTORING
+			if (G_UNLIKELY(g_array_index(master->bflang->st->contexts, Tcontext, contextnum).autocomplete_has_conditions)) {
+				if (found==NULL) {
+					found = get_foundcache_at_offset(btv, gtk_text_iter_get_offset(&cursorpos), NULL);
+				}
+				items = process_conditional_items(btv, found, contextnum, items);
+				free_items=TRUE;
+			}
 			{
 				GCompletion *compl = identifier_ac_get_completion(master, contextnum, FALSE);
 				DBG_IDENTIFIER("got completion %p for context %d\n", compl, contextnum);
@@ -684,7 +773,6 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 						g_free(newprefix2);
 				}
 			}
-#endif
 		}
 		if (closetag || items2
 			|| (items != NULL && (items->next != NULL || strcmp(items->data, prefix) != 0))) {
@@ -714,24 +802,32 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 			acwin_fill_tree(ACWIN(btv->autocomp), items, items2, closetag, !below, numitems);
 			gtk_widget_show(ACWIN(btv->autocomp)->win);
 			selection = gtk_tree_view_get_selection(ACWIN(btv->autocomp)->tree);
-			if (below)
+			if (below) {
+				DBG_AUTOCOMP("autocomp_run, popup-below, get first iter for selection\n");
 				gtk_tree_model_get_iter_first(GTK_TREE_MODEL(ACWIN(btv->autocomp)->store), &it);
-			else {
+			} else {
 				GtkTreePath *path;
+				DBG_AUTOCOMP("autocomp_run, popup-above, select last iter and scroll max down\n");
 				gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(ACWIN(btv->autocomp)->store), &it, NULL,
 											  gtk_tree_model_iter_n_children(GTK_TREE_MODEL
 																			 (ACWIN(btv->autocomp)->store),
 																			 NULL) - 1);
 				path = gtk_tree_model_get_path(GTK_TREE_MODEL(ACWIN(btv->autocomp)->store), &it);
-				gtk_tree_view_scroll_to_cell(ACWIN(btv->autocomp)->tree, path, NULL, FALSE, 0.0, 0.0);
+				gtk_tree_view_scroll_to_cell(ACWIN(btv->autocomp)->tree, path, NULL, FALSE, 1.0, 1.0);
 				gtk_tree_path_free(path);
 			}
+			/* this forces that selection changed will be called two lines below*/
+			gtk_tree_selection_unselect_all(selection);
 			ACWIN(btv->autocomp)->in_fill=FALSE;
+			DBG_AUTOCOMP("call select_iter on autocomp window\n");
 			gtk_tree_selection_select_iter(selection, &it);
 			g_free(closetag);
 		} else {
 			acwin_cleanup(btv);
 		}
+		if (free_items) {
+			g_list_free(items);
+		}
 		g_free(newprefix);
 		g_free(prefix);
 	} else {
