@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bftextview2_patcompile.c
  *
- * Copyright (C) 2008,2009,2011,2013,2014 Olivier Sessink
+ * Copyright (C) 2008,2009,2011,2013,2014,2015 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -637,8 +637,14 @@ compile_keyword_to_DFA(Tscantable * st, const gchar * keyword, guint16 matchnum,
 	g_free(pattern);
 }
 
+/*
+ * symbols can be free'ed
+ * contexthighlight is directly stored in Tcontext
+ *
+ *
+ */
 gint16
-new_context(Tscantable * st, guint expected_size, const gchar * lang, const gchar * symbols, const gchar * contexthighlight,
+new_context(Tscantable * st, guint expected_size, const gchar * symbols, const gchar * contexthighlight,
 			gboolean autocomplete_case_insens, gboolean default_spellcheck, gboolean dump_dfa_run)
 {
 	gint16 context;
@@ -690,12 +696,18 @@ void
 match_autocomplete_reference(Tscantable * st, guint16 matchnum, guint16 context)
 {
 	gint pattern_id = matchnum;
+	gboolean has_reference;
+
+	has_reference = (g_array_index(st->matches, Tpattern, matchnum).reference!=NULL);
+
 	if (!g_array_index(st->contexts, Tcontext, context).patternhash) {
 		g_array_index(st->contexts, Tcontext, context).patternhash =
 			g_hash_table_new(g_str_hash, g_str_equal);
 	}
-	g_hash_table_insert(g_array_index(st->contexts, Tcontext, context).patternhash,
+	if (has_reference) {
+		g_hash_table_insert(g_array_index(st->contexts, Tcontext, context).patternhash,
 						g_array_index(st->matches, Tpattern, matchnum).pattern, GINT_TO_POINTER(pattern_id));
+	}
 
 	if (g_array_index(st->matches, Tpattern, matchnum).tagclose_from_blockstack
 		&& !g_array_index(st->contexts, Tcontext, context).has_tagclose_from_blockstack) {
@@ -703,13 +715,6 @@ match_autocomplete_reference(Tscantable * st, guint16 matchnum, guint16 context)
 		g_array_index(st->contexts, Tcontext, context).has_tagclose_from_blockstack = 1;
 	}
 
-/*	if (g_array_index(st->matches, Tpattern, matchnum).reference && !g_array_index(st->matches, Tpattern, matchnum).is_regex) {
-		if (!g_array_index(st->contexts, Tcontext, context).reference) {
-			DBG_PATCOMPILE("create hashtable for context %d\n",context);
-			g_array_index(st->contexts, Tcontext, context).reference = g_hash_table_new(g_str_hash,g_str_equal);
-		}
-		g_hash_table_insert(g_array_index(st->contexts, Tcontext, context).reference,g_array_index(st->matches, Tpattern, matchnum).pattern,g_array_index(st->matches, Tpattern, matchnum).reference);
-	}*/
 	if (g_array_index(st->matches, Tpattern, matchnum).autocomp_items) {
 		GSList *tmpslist = g_array_index(st->matches, Tpattern, matchnum).autocomp_items;
 		GList *list = NULL;
@@ -718,54 +723,80 @@ match_autocomplete_reference(Tscantable * st, guint16 matchnum, guint16 context)
 			g_array_index(st->contexts, Tcontext, context).ac = g_completion_new(NULL);
 			if (g_array_index(st->contexts, Tcontext, context).autocomplete_case_insens)
 				g_completion_set_compare(g_array_index(st->contexts, Tcontext, context).ac, strncasecmp);
+			g_array_index(st->contexts, Tcontext, context).autocomplete_has_conditions = 0; /* defaults to 0 */
 		}
-
 		while (tmpslist) {
+			gboolean already_handled=FALSE;
 			Tpattern_autocomplete *pac = tmpslist->data;
-			list = g_list_prepend(list, pac->autocomplete_string);
-			g_hash_table_insert(g_array_index(st->contexts, Tcontext, context).patternhash,
-								pac->autocomplete_string, GINT_TO_POINTER(pattern_id));
+			if (pac->condition != 0) {
+				g_array_index(st->contexts, Tcontext, context).autocomplete_has_conditions = 1;
+
+			/* there is a special situation in Bluefish, if a attribute-context is re-used, a new conditional item
+			is added to the '>' pattern, and match_autocomplete_reference() is called AGAIN for the same pattern.
+			we can detect this by looking it up in the hash table  */
+				if (GPOINTER_TO_INT(g_hash_table_lookup(g_array_index(st->contexts, Tcontext, context).patternhash,pac->autocomplete_string))==pattern_id) {
+					DBG_AUTOCOMP("match_autocomplete_reference, ignore autocomplete string %s, is already handled\n",pac->autocomplete_string);
+					already_handled = TRUE;
+				}
+			}
+#ifdef DEVELOPMENT
+			if (g_strcmp0(pac->autocomplete_string, g_array_index(st->matches, Tpattern, matchnum).pattern)!=0) {
+				gint hashpat = GPOINTER_TO_INT(g_hash_table_lookup(g_array_index(st->contexts, Tcontext, context).patternhash,pac->autocomplete_string));
+				if (hashpat && !already_handled) {
+					g_print("autocompletion item %s is already in the context %d hash table for pattern %d, now handling pattern %d that has %d autocomplete entries?!\n", pac->autocomplete_string, context, hashpat, pattern_id, g_slist_length(g_array_index(st->matches, Tpattern, matchnum).autocomp_items));
+				}
+			}
+#endif
+			if (!already_handled) {
+				list = g_list_prepend(list, pac->autocomplete_string);
+				/* we only need to add an autocomplete string to the hash table if the pattern has a
+				reference text, or backup_cursor is set, or trigger_new_autocomp_popup, or there
+				is a condition AND if the autocompletion patern is different than the pattern itself (which is
+				added to the hash table earlier in this function) */
+				if (g_strcmp0(pac->autocomplete_string, g_array_index(st->matches, Tpattern, matchnum).pattern)!=0 && (has_reference || pac->condition != 0 || pac->autocomplete_backup_cursor!=0 || pac->trigger_new_autocomp_popup!=0)) {
+					DBG_AUTOCOMP("match_autocomplete_reference, enter autocompletion item %s in hash table\n",pac->autocomplete_string);
+					g_hash_table_insert(g_array_index(st->contexts, Tcontext, context).patternhash,
+									pac->autocomplete_string, GINT_TO_POINTER(pattern_id));
+				}
+			}
 			tmpslist = g_slist_next(tmpslist);
 		}
 		g_completion_add_items(g_array_index(st->contexts, Tcontext, context).ac, list);
 		g_list_free(list);
 	}
-#ifdef OLD_AUTOCOMP
-	if (g_array_index(st->matches, Tpattern, matchnum).autocomplete) {
-		GList *list;
-		gchar *tmp;
-		if (!g_array_index(st->contexts, Tcontext, context).ac) {
-			DBG_PATCOMPILE("create g_completion for context %d\n", context);
-			g_array_index(st->contexts, Tcontext, context).ac = g_completion_new(NULL);
-			if (g_array_index(st->contexts, Tcontext, context).autocomplete_case_insens)
-				g_completion_set_compare(g_array_index(st->contexts, Tcontext, context).ac, strncasecmp);
-		}
-		if (g_array_index(st->matches, Tpattern, matchnum).autocomplete_string) {
-			tmp = g_array_index(st->matches, Tpattern, matchnum).autocomplete_string;
-		} else {
-			tmp = g_array_index(st->matches, Tpattern, matchnum).pattern;
-		}
-
-		list = g_list_prepend(NULL, tmp);
-		g_completion_add_items(g_array_index(st->contexts, Tcontext, context).ac, list);
-		DBG_AUTOCOMP("adding %s to GCompletion\n", (gchar *) list->data);
-		g_list_free(list);
-		if (g_array_index(st->matches, Tpattern, matchnum).autocomplete_string) {
-			/*if (g_array_index(st->matches, Tpattern, matchnum).reference) {
-			   g_hash_table_insert(g_array_index(st->contexts, Tcontext, context).reference,g_array_index(st->matches, Tpattern, matchnum).autocomplete_string,g_array_index(st->matches, Tpattern, matchnum).reference);
-			   } */
-			g_hash_table_insert(g_array_index(st->contexts, Tcontext, context).patternhash,
-								g_array_index(st->matches, Tpattern, matchnum).autocomplete_string,
-								GINT_TO_POINTER(pattern_id));
-		}
+}
+/*
+ * refname should not be freed, it is set in Tpattern_condition
+ *
+ */
+static guint16
+add_condition(Tscantable * st, const gchar *refname, gint relation, gint mode)
+{
+	guint16 condnum = st->conditions->len;
+	DBG_PARSING("add_condition, condnum=%d, called with refname %s\n",condnum,refname);
+	g_array_set_size(st->conditions, st->conditions->len + 1);
+	if (mode < 1 || mode > 4) {
+		g_warning("Error in language file: condition mode %d is not defined\n",mode);
+		return 0;
 	}
-#endif
+	g_array_index(st->conditions, Tpattern_condition, condnum).refname = g_strdup(refname);
+	g_array_index(st->conditions, Tpattern_condition, condnum).parentrelation = relation;
+	g_array_index(st->conditions, Tpattern_condition, condnum).relationtype = mode;
+	return condnum;
 }
 
-
+/*
+ * autocomplete_string should be free'ed
+ * autocomplete_append should be free'ed
+ * refname should be free'ed
+ *
+ *
+ */
 void
 match_add_autocomp_item(Tscantable * st, guint16 matchnum, const gchar * autocomplete_string,
-						const gchar * autocomplete_append, guint8 autocomplete_backup_cursor)
+						const gchar * autocomplete_append, guint8 autocomplete_backup_cursor,
+						guint8 trigger_new_autocomp_popup,
+						const gchar *refname, gint relation, gint mode)
 {
 	Tpattern_autocomplete *pac = g_slice_new(Tpattern_autocomplete);
 	if (autocomplete_string) {
@@ -789,6 +820,8 @@ match_add_autocomp_item(Tscantable * st, guint16 matchnum, const gchar * autocom
 	}
 #endif
 	pac->autocomplete_backup_cursor = autocomplete_backup_cursor;
+	pac->trigger_new_autocomp_popup = trigger_new_autocomp_popup;
+	pac->condition = (mode==0) ? 0 : add_condition(st, refname, relation, mode);
 	g_array_index(st->matches, Tpattern, matchnum).autocomp_items =
 		g_slist_prepend(g_array_index(st->matches, Tpattern, matchnum).autocomp_items, pac);
 }
@@ -815,25 +848,15 @@ compile_existing_match(Tscantable * st, guint16 matchnum, gint16 context, gpoint
 	}
 	match_autocomplete_reference(st, matchnum, context);
 }
-#ifdef CONDITIONALPATTERN
 
 void
 pattern_set_condition(Tscantable * st, guint16 matchnum, gchar *refname, gint relation, gint mode)
 {
-	guint16 condnum = st->conditions->len;
-	DBG_PARSING("pattern_set_condition, condnum=%d, called with refname %s for matchnum %d\n",condnum,refname,matchnum);
-	g_array_set_size(st->conditions, st->conditions->len + 1);
-	if (mode < 1 || mode > 4) {
-		g_warning("Error in language file: condition mode %d is not defined\n",mode);
-		return;
-	}
-	g_array_index(st->conditions, Tpattern_condition, condnum).refname = g_strdup(refname);
-	g_array_index(st->conditions, Tpattern_condition, condnum).parentrelation = relation;
-	g_array_index(st->conditions, Tpattern_condition, condnum).relationtype = mode;
+	guint16 condnum = add_condition(st, refname, relation, mode);
 	DBG_PARSING("pattern %d (%s) now has condition %d\n",matchnum,g_array_index(st->matches, Tpattern, matchnum).pattern,condnum);
 	g_array_index(st->matches, Tpattern, matchnum).condition = condnum;
 }
-#endif /*CONDITIONALPATTERN*/
+
 void
 pattern_set_blockmatch(Tscantable * st, guint16 matchnum,
 							gboolean starts_block,
