@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * file_dialogs.c - file dialogs
  *
- * Copyright (C) 2005-2014 Olivier Sessink
+ * Copyright (C) 2005-2015 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -825,7 +825,7 @@ doc_save_backend(Tdocument * doc, Tdocsave_mode savemode, gboolean close_doc,
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(doc->view), FALSE);
 	DEBUG_MSG("doc_save_backend, calling file_checkNsave_uri_async with uri %p for %zd bytes\n", dest_uri, strlen(buffer->data));
 	doc->save =
-		file_checkNsave_uri_async(dest_uri, dest_finfo, buffer, numbytes, savemode == docsave_normal,
+		file_checkNsave_uri_async(dest_uri, dest_finfo, buffer, numbytes, (savemode == docsave_normal && main_v->props.check_for_modified_on_disk!=0),
 								  main_v->props.backup_file, doc_checkNsave_lcb, dsb);
 
 	if (firstsave || savemode == docsave_saveas || savemode == docsave_move) {
@@ -1204,9 +1204,13 @@ file_new_doc(Tbfwin * bfwin)
 
 	if (bfwin->session->template && bfwin->session->template[0]) {
 		template = g_file_new_for_commandline_arg(bfwin->session->template);
+		bfwin->focus_next_new_doc = TRUE;
 	}
 	doc = doc_new_with_template(bfwin, template, TRUE);
-	bfwin_switch_to_document_by_pointer(bfwin, doc);
+	if (!template && doc != bfwin->current_document) {
+		/* for a template this will be done by the template loader callback */
+		bfwin_switch_to_document_by_pointer(bfwin, doc);
+	}
 }
 
 /**
@@ -1221,13 +1225,7 @@ file_new_doc(Tbfwin * bfwin)
 void
 file_new_cb(GtkWidget * widget, Tbfwin * bfwin)
 {
-	Tdocument *doc;
-	GFile *template = NULL;
-	if (bfwin->session->template && bfwin->session->template[0]) {
-		template = g_file_new_for_commandline_arg(bfwin->session->template);
-	}
-	doc = doc_new_with_template(bfwin, template, TRUE);
-	bfwin_switch_to_document_by_pointer(bfwin, doc);
+	file_new_doc(bfwin);
 }
 
 static void
@@ -1552,10 +1550,10 @@ modified_on_disk_check_lcb(gpointer data)
 void
 modified_on_disk_check_init(void)
 {
-	if (main_v->props.do_periodic_check && !main_v->periodic_check_id)
+	if (main_v->props.check_for_modified_on_disk==1 && !main_v->periodic_check_id)
 		main_v->periodic_check_id =
 			g_timeout_add_seconds_full(G_PRIORITY_LOW, 15, modified_on_disk_check_lcb, NULL, NULL);
-	else if (!main_v->props.do_periodic_check && main_v->periodic_check_id) {
+	else if (main_v->props.check_for_modified_on_disk!=1 && main_v->periodic_check_id) {
 		g_source_remove(main_v->periodic_check_id);
 		main_v->periodic_check_id = 0;
 	}
